package test.compiled.example13_send_receive;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class ReceiverBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Start = 1;

	public static final short Finish = 2;

	private final BroadcastChannel p;

	public ReceiverBehavior(SymbolsTable symbols, BroadcastChannel p) {
		super("receiver", idSeed++, symbols);
		this.p = p;
	}

	private final String[] stateNames = new String[] {"start", "finish"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return p.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ReceiverConfiguration me = (ReceiverConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ReceiverConfiguration me = (ReceiverConfiguration) configuration.behaviorConfigurations[id];
			p.input(id, actions);
			if ( p.clockId != -1 ) {
				DBM.set(configuration.dbm, p.clockId, 0);
			}
			if ((me.x) < (10)) {
				me.x = (me.x) + (1);
				me.state = Start;
				if (true) return true;
			} else {
				me.state = Finish;
				if (true) return true;
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ReceiverConfiguration me = new ReceiverConfiguration();
		//Initial action code.
		me.x = 0;
		me.state = Start;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ReceiverConfiguration me = (ReceiverConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ReceiverBehavior.Start: 
			text.append("@start,");
			break;
		case ReceiverBehavior.Finish: 
			text.append("@finish,");
			break;
		}
		text.append("\n\t- x=");
		text.append(Integer.toString(me.x));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ReceiverConfiguration me = (ReceiverConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ReceiverBehavior.Start: 
			text.append("@start,");
			break;
		case ReceiverBehavior.Finish: 
			text.append("@finish,");
			break;
		}
		text.append(" x=");
		text.append(Integer.toString(me.x));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ReceiverConfiguration me = new ReceiverConfiguration();
		me.state = boost.readShort();
		me.x = boost.readInt();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ReceiverConfiguration me = (ReceiverConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.x);
	}

}

