package test.compiled.example16_local_mutex;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class PBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short AccessS = 1;

	public static final short StartS = 2;

	public static final short ReleaseS = 3;

	public static final short EndS = 4;

	public PBehavior(SymbolsTable symbols) {
		super("p", idSeed++, symbols);
	}

	private final String[] stateNames = new String[] {"accessS", "startS", "releaseS", "endS"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != StartS) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			me.state = AccessS;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != AccessS) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			me.state = ReleaseS;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != ReleaseS) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			me.state = EndS;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition3 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != EndS) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			me.state = EndS;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
		transition3,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		PConfiguration me = new PConfiguration();
		//Initial action code.
		me.state = StartS;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		PConfiguration me = (PConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case PBehavior.AccessS: 
			text.append("@accessS,");
			break;
		case PBehavior.StartS: 
			text.append("@startS,");
			break;
		case PBehavior.ReleaseS: 
			text.append("@releaseS,");
			break;
		case PBehavior.EndS: 
			text.append("@endS,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		PConfiguration me = (PConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case PBehavior.AccessS: 
			text.append("@accessS,");
			break;
		case PBehavior.StartS: 
			text.append("@startS,");
			break;
		case PBehavior.ReleaseS: 
			text.append("@releaseS,");
			break;
		case PBehavior.EndS: 
			text.append("@endS,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		PConfiguration me = new PConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		PConfiguration me = (PConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

