package test.compiled.example1_rw;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ReferencedArgumentInt;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class ProcWBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Start = 1;

	private final BroadcastChannel w_port;

	private final ReferencedArgumentInt dataRef;

	public ProcWBehavior(SymbolsTable symbols, BroadcastChannel w_port, ReferencedArgumentInt data) {
		super("ProcW", idSeed++, symbols);
		this.w_port = w_port;
		this.dataRef = data;
	}

	private final String[] stateNames = new String[] {"Start"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return w_port.id;
		}

		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProcWConfiguration me = (ProcWConfiguration) configuration.behaviorConfigurations[id];
			int data = dataRef.get(configuration);
			modifiedBehaviors[dataRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcWConfiguration me = (ProcWConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcWConfiguration me = (ProcWConfiguration) configuration.behaviorConfigurations[id];
			int data = dataRef.get(configuration);
			try {
				w_port.input(id, actions);
				if ( w_port.clockId != -1 ) {
					DBM.set(configuration.dbm, w_port.clockId, 0);
				}
				data = me.val;
				me.val = ((me.val) + (1)) % (10);
				me.state = Start;
				if (true) return true;
			} finally {
				dataRef.set(configuration, data);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ProcWConfiguration me = new ProcWConfiguration();
		//Initial action code.
		me.val = 0;
		me.state = Start;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProcWConfiguration me = (ProcWConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProcWBehavior.Start: 
			text.append("@Start,");
			break;
		}
		text.append("\n\t- val=");
		text.append(Integer.toString(me.val));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProcWConfiguration me = (ProcWConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProcWBehavior.Start: 
			text.append("@Start,");
			break;
		}
		text.append(" val=");
		text.append(Integer.toString(me.val));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ProcWConfiguration me = new ProcWConfiguration();
		me.state = boost.readShort();
		me.val = boost.readInt();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ProcWConfiguration me = (ProcWConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.val);
	}

}

