package test.compiled.example1_rw;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class SchedBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short S1 = 1;

	public static final short S0 = 2;

	private final BroadcastChannel r;

	private final BroadcastChannel w;

	public SchedBehavior(SymbolsTable symbols, BroadcastChannel r, BroadcastChannel w) {
		super("Sched", idSeed++, symbols);
		this.r = r;
		this.w = w;
	}

	private final String[] stateNames = new String[] {"s1", "s0"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return r.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			SchedConfiguration me = (SchedConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != S0) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final SchedConfiguration me = (SchedConfiguration) configuration.behaviorConfigurations[id];
			r.output(id, actions);
			me.state = S1;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return w.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			SchedConfiguration me = (SchedConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != S1) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final SchedConfiguration me = (SchedConfiguration) configuration.behaviorConfigurations[id];
			w.output(id, actions);
			me.state = S0;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		SchedConfiguration me = new SchedConfiguration();
		//Initial action code.
		me.state = S0;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		SchedConfiguration me = (SchedConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case SchedBehavior.S1: 
			text.append("@s1,");
			break;
		case SchedBehavior.S0: 
			text.append("@s0,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		SchedConfiguration me = (SchedConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case SchedBehavior.S1: 
			text.append("@s1,");
			break;
		case SchedBehavior.S0: 
			text.append("@s0,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		SchedConfiguration me = new SchedConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		SchedConfiguration me = (SchedConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

