package test.compiled.example3_async_sync;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.FiacreQueueUtil;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;

@SuppressWarnings("all")
public class Fifo4blockBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Start = 1;

	private final BroadcastChannel put;

	private final BroadcastChannel get;

	public Fifo4blockBehavior(SymbolsTable symbols, BroadcastChannel put, BroadcastChannel get) {
		super("Fifo4block", idSeed++, symbols);
		this.put = put;
		this.get = get;
	}

	private final String[] stateNames = new String[] {"Start"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return put.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			Fifo4blockConfiguration me = (Fifo4blockConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			if ( ( !( (me.fifo.length >= 4 ))) == false) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final Fifo4blockConfiguration me = (Fifo4blockConfiguration) configuration.behaviorConfigurations[id];
			int localV = 0;
			put.input(id, actions);
			localV = put.value(int.class,0);
			if ( put.clockId != -1 ) {
				DBM.set(configuration.dbm, put.clockId, 0);
			}
			me.fifo = FiacreQueueUtil.enqueue(me.fifo,localV);
			me.state = Start;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			Fifo4blockConfiguration me = (Fifo4blockConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			if ( ( !( !( (me.fifo.length >= 4 )))) == false) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final Fifo4blockConfiguration me = (Fifo4blockConfiguration) configuration.behaviorConfigurations[id];
			me.state = Start;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return get.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			Fifo4blockConfiguration me = (Fifo4blockConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			if ( ( !( (me.fifo.length == 0))) == false) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final Fifo4blockConfiguration me = (Fifo4blockConfiguration) configuration.behaviorConfigurations[id];
			int localV = 0;
			localV =  FiacreQueueUtil.first(me.fifo);
			me.fifo =  FiacreQueueUtil.dequeue(me.fifo);
			get.output(id, actions, localV);
			me.state = Start;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition3 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			Fifo4blockConfiguration me = (Fifo4blockConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			if ( ( !( !( (me.fifo.length == 0)))) == false) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final Fifo4blockConfiguration me = (Fifo4blockConfiguration) configuration.behaviorConfigurations[id];
			me.state = Start;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
		transition3,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		Fifo4blockConfiguration me = new Fifo4blockConfiguration();
		int localV;
		//Initial action code.
		me.state = Start;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		Fifo4blockConfiguration me = (Fifo4blockConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case Fifo4blockBehavior.Start: 
			text.append("@Start,");
			break;
		}
		text.append("\n\t- fifo=");
		text.append(Arrays.toString(me.fifo));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		Fifo4blockConfiguration me = (Fifo4blockConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case Fifo4blockBehavior.Start: 
			text.append("@Start,");
			break;
		}
		text.append(" fifo=");
		text.append(Arrays.toString(me.fifo));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		Fifo4blockConfiguration me = new Fifo4blockConfiguration();
		me.state = boost.readShort();
		me.fifo = new int[4];
		me.fifo = new int[boost.readInt()];
		for (int i=0; i<me.fifo.length; i++) {
			me.fifo[i] = boost.readInt();
		}
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		Fifo4blockConfiguration me = (Fifo4blockConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.fifo.length);
		for (int i=0; i<me.fifo.length; i++) {
			boost.writeInt(me.fifo[i]);
		}
	}

}

