package test.compiled.example4_comput;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.DelayAction;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.WaitChannel;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;

@SuppressWarnings("all")
public class ComputeBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Start = 1;

	public static final short Compute = 2;

	public static final short Done = 3;

	private final BroadcastChannel req;

	private final BroadcastChannel done;

	private final WaitChannel wait0;

	private final Channel[] channels;

	public void collectChannels(List<Channel> result) {
		result.addAll(Arrays.asList(channels));
	}

	public ComputeBehavior(SymbolsTable symbols, BroadcastChannel req, BroadcastChannel done) {
		super("compute", idSeed++, symbols);
		this.req = req;
		this.done = done;
		symbols.registerClockIndexes(this.getId(), (short) 1);
		
		wait0 = new WaitChannel(null, symbols, this.getId(), (short) 0, 11, false, 11, false);
		channels = new Channel[] { wait0,  };
	}

	private final String[] stateNames = new String[] {"Start", "compute", "Done"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return req.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ComputeConfiguration me = (ComputeConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ComputeConfiguration me = (ComputeConfiguration) configuration.behaviorConfigurations[id];
			req.input(id, actions);
			if ( req.clockId != -1 ) {
				DBM.set(configuration.dbm, req.clockId, 0);
			}
			me.state = Compute;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return wait0.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ComputeConfiguration me = (ComputeConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Compute) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ComputeConfiguration me = (ComputeConfiguration) configuration.behaviorConfigurations[id];
			DBM.set(configuration.dbm, wait0.clockId, 0);
			actions.add(new DelayAction(id));
			me.state = Done;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return done.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ComputeConfiguration me = (ComputeConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Done) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ComputeConfiguration me = (ComputeConfiguration) configuration.behaviorConfigurations[id];
			done.output(id, actions);
			me.state = Start;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ComputeConfiguration me = new ComputeConfiguration();
		//Initial action code.
		me.state = Start;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ComputeConfiguration me = (ComputeConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ComputeBehavior.Start: 
			text.append("@Start,");
			break;
		case ComputeBehavior.Compute: 
			text.append("@compute,");
			break;
		case ComputeBehavior.Done: 
			text.append("@Done,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ComputeConfiguration me = (ComputeConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ComputeBehavior.Start: 
			text.append("@Start,");
			break;
		case ComputeBehavior.Compute: 
			text.append("@compute,");
			break;
		case ComputeBehavior.Done: 
			text.append("@Done,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ComputeConfiguration me = new ComputeConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ComputeConfiguration me = (ComputeConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

