package test.compiled.example5_prodcons_nomulti;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.FiacreQueueUtil;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;

@SuppressWarnings("all")
public class FifoBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Start = 1;

	public static final short Fwait = 2;

	public static final short Error = 3;

	public static final short Stop = 4;

	private final BroadcastChannel sEvt;

	private final BroadcastChannel eEvt;

	private final BroadcastChannel rcv;

	private final BroadcastChannel snd;

	public FifoBehavior(SymbolsTable symbols, BroadcastChannel sEvt, BroadcastChannel eEvt, BroadcastChannel rcv, BroadcastChannel snd) {
		super("fifo", idSeed++, symbols);
		this.sEvt = sEvt;
		this.eEvt = eEvt;
		this.rcv = rcv;
		this.snd = snd;
	}

	private final String[] stateNames = new String[] {"start", "fwait", "error", "stop"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return sEvt.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			sEvt.input(id, actions);
			if ( sEvt.clockId != -1 ) {
				DBM.set(configuration.dbm, sEvt.clockId, 0);
			}
			me.state = Fwait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Fwait) return false;
			if ( ( (me.fifo.length >= 10 )) == false) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			me.state = Fwait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return rcv.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Fwait) return false;
			if ( ( !( (me.fifo.length >= 10 ))) == false) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			rcv.input(id, actions);
			me.data = rcv.value(int.class,0);
			if ( rcv.clockId != -1 ) {
				DBM.set(configuration.dbm, rcv.clockId, 0);
			}
			me.fifo = FiacreQueueUtil.enqueue(me.fifo,me.data);
			me.state = Fwait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition3 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Fwait) return false;
			if ( ( (me.fifo.length == 0)) == false) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			me.state = Fwait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition4 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return snd.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Fwait) return false;
			if ( ( !( (me.fifo.length == 0))) == false) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			me.data =  FiacreQueueUtil.first(me.fifo);
			me.fifo =  FiacreQueueUtil.dequeue(me.fifo);
			snd.output(id, actions, me.data);
			me.state = Fwait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition5 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return sEvt.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Fwait) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			sEvt.input(id, actions);
			if ( sEvt.clockId != -1 ) {
				DBM.set(configuration.dbm, sEvt.clockId, 0);
			}
			me.state = Error;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition6 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return eEvt.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Fwait) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			eEvt.input(id, actions);
			if ( eEvt.clockId != -1 ) {
				DBM.set(configuration.dbm, eEvt.clockId, 0);
			}
			me.state = Stop;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition7 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Stop) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			me.state = Stop;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition8 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Error) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final FifoConfiguration me = (FifoConfiguration) configuration.behaviorConfigurations[id];
			me.state = Error;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
		transition3,
		transition4,
		transition5,
		transition6,
		transition7,
		transition8,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		FifoConfiguration me = new FifoConfiguration();
		//Initial action code.
		me.state = Start;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		FifoConfiguration me = (FifoConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case FifoBehavior.Start: 
			text.append("@start,");
			break;
		case FifoBehavior.Fwait: 
			text.append("@fwait,");
			break;
		case FifoBehavior.Error: 
			text.append("@error,");
			break;
		case FifoBehavior.Stop: 
			text.append("@stop,");
			break;
		}
		text.append("\n\t- data=");
		text.append(Integer.toString(me.data));
		text.append("\n\t- fifo=");
		text.append(Arrays.toString(me.fifo));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		FifoConfiguration me = (FifoConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case FifoBehavior.Start: 
			text.append("@start,");
			break;
		case FifoBehavior.Fwait: 
			text.append("@fwait,");
			break;
		case FifoBehavior.Error: 
			text.append("@error,");
			break;
		case FifoBehavior.Stop: 
			text.append("@stop,");
			break;
		}
		text.append(" data=");
		text.append(Integer.toString(me.data));
		text.append(" fifo=");
		text.append(Arrays.toString(me.fifo));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		FifoConfiguration me = new FifoConfiguration();
		me.state = boost.readShort();
		me.data = boost.readInt();
		me.fifo = new int[Example5_prodCons_nomultiRoot.FIFO_SIZE];
		me.fifo = new int[boost.readInt()];
		for (int i=0; i<me.fifo.length; i++) {
			me.fifo[i] = boost.readInt();
		}
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		FifoConfiguration me = (FifoConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.data);
		boost.writeInt(me.fifo.length);
		for (int i=0; i<me.fifo.length; i++) {
			boost.writeInt(me.fifo[i]);
		}
	}

}

