package test.compiled.example5_prodcons_nomulti;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.DelayAction;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.WaitChannel;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;

@SuppressWarnings("all")
public class OsProcessBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short SWait = 1;

	public static final short Start = 2;

	public static final short Stop = 3;

	public static final short EWait = 4;

	private final BroadcastChannel startMMI;

	private final BroadcastChannel stopMMI;

	private final WaitChannel wait0;

	private final WaitChannel wait1;

	private final Channel[] channels;

	public void collectChannels(List<Channel> result) {
		result.addAll(Arrays.asList(channels));
	}

	public OsProcessBehavior(SymbolsTable symbols, BroadcastChannel startMMI, BroadcastChannel stopMMI) {
		super("osProcess", idSeed++, symbols);
		this.startMMI = startMMI;
		this.stopMMI = stopMMI;
		symbols.registerClockIndexes(this.getId(), (short) 1);
		
		wait0 = new WaitChannel(null, symbols, this.getId(), (short) 0, 100, false, 200, false);
		wait1 = new WaitChannel(null, symbols, this.getId(), (short) 0, 100, false, 200, false);
		channels = new Channel[] { wait0, wait1,  };
	}

	private final String[] stateNames = new String[] {"sWait", "start", "stop", "eWait"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return startMMI.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			OsProcessConfiguration me = (OsProcessConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final OsProcessConfiguration me = (OsProcessConfiguration) configuration.behaviorConfigurations[id];
			startMMI.output(id, actions);
			me.state = SWait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return wait0.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			OsProcessConfiguration me = (OsProcessConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != SWait) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final OsProcessConfiguration me = (OsProcessConfiguration) configuration.behaviorConfigurations[id];
			DBM.set(configuration.dbm, wait0.clockId, 0);
			actions.add(new DelayAction(id));
			me.state = Stop;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return stopMMI.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			OsProcessConfiguration me = (OsProcessConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Stop) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final OsProcessConfiguration me = (OsProcessConfiguration) configuration.behaviorConfigurations[id];
			stopMMI.output(id, actions);
			me.state = EWait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition3 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return wait1.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			OsProcessConfiguration me = (OsProcessConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != EWait) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final OsProcessConfiguration me = (OsProcessConfiguration) configuration.behaviorConfigurations[id];
			DBM.set(configuration.dbm, wait1.clockId, 0);
			actions.add(new DelayAction(id));
			me.state = Start;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
		transition3,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		OsProcessConfiguration me = new OsProcessConfiguration();
		//Initial action code.
		me.state = Start;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		OsProcessConfiguration me = (OsProcessConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case OsProcessBehavior.SWait: 
			text.append("@sWait,");
			break;
		case OsProcessBehavior.Start: 
			text.append("@start,");
			break;
		case OsProcessBehavior.Stop: 
			text.append("@stop,");
			break;
		case OsProcessBehavior.EWait: 
			text.append("@eWait,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		OsProcessConfiguration me = (OsProcessConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case OsProcessBehavior.SWait: 
			text.append("@sWait,");
			break;
		case OsProcessBehavior.Start: 
			text.append("@start,");
			break;
		case OsProcessBehavior.Stop: 
			text.append("@stop,");
			break;
		case OsProcessBehavior.EWait: 
			text.append("@eWait,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		OsProcessConfiguration me = new OsProcessConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		OsProcessConfiguration me = (OsProcessConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

