package test.compiled.example5_prodcons_nomulti;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class ProducerBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Start = 1;

	public static final short Produce = 2;

	public static final short Stop = 3;

	public static final short Error = 4;

	private final BroadcastChannel sEvt;

	private final BroadcastChannel eEvt;

	private final BroadcastChannel put;

	public ProducerBehavior(SymbolsTable symbols, BroadcastChannel sEvt, BroadcastChannel eEvt, BroadcastChannel put) {
		super("producer", idSeed++, symbols);
		this.sEvt = sEvt;
		this.eEvt = eEvt;
		this.put = put;
	}

	private final String[] stateNames = new String[] {"start", "produce", "stop", "error"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return sEvt.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			sEvt.input(id, actions);
			if ( sEvt.clockId != -1 ) {
				DBM.set(configuration.dbm, sEvt.clockId, 0);
			}
			me.state = Produce;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return eEvt.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			eEvt.input(id, actions);
			if ( eEvt.clockId != -1 ) {
				DBM.set(configuration.dbm, eEvt.clockId, 0);
			}
			me.state = Stop;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return put.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Produce) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			put.output(id, actions, me.data);
			me.data = (me.data) + (1);
			if ((me.data) >= (Example5_prodCons_nomultiRoot.N)) {
				me.state = Stop;
				if (true) return true;
			} else {
				me.state = Produce;
				if (true) return true;
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition3 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return sEvt.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Produce) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			sEvt.input(id, actions);
			if ( sEvt.clockId != -1 ) {
				DBM.set(configuration.dbm, sEvt.clockId, 0);
			}
			me.state = Error;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition4 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return eEvt.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Produce) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			eEvt.input(id, actions);
			if ( eEvt.clockId != -1 ) {
				DBM.set(configuration.dbm, eEvt.clockId, 0);
			}
			me.state = Stop;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition5 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Stop) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			me.state = Stop;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition6 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Error) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			me.state = Error;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
		transition3,
		transition4,
		transition5,
		transition6,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ProducerConfiguration me = new ProducerConfiguration();
		//Initial action code.
		me.data = 0;
		me.state = Start;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProducerConfiguration me = (ProducerConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProducerBehavior.Start: 
			text.append("@start,");
			break;
		case ProducerBehavior.Produce: 
			text.append("@produce,");
			break;
		case ProducerBehavior.Stop: 
			text.append("@stop,");
			break;
		case ProducerBehavior.Error: 
			text.append("@error,");
			break;
		}
		text.append("\n\t- data=");
		text.append(Integer.toString(me.data));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProducerConfiguration me = (ProducerConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProducerBehavior.Start: 
			text.append("@start,");
			break;
		case ProducerBehavior.Produce: 
			text.append("@produce,");
			break;
		case ProducerBehavior.Stop: 
			text.append("@stop,");
			break;
		case ProducerBehavior.Error: 
			text.append("@error,");
			break;
		}
		text.append(" data=");
		text.append(Integer.toString(me.data));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ProducerConfiguration me = new ProducerConfiguration();
		me.state = boost.readShort();
		me.data = boost.readInt();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ProducerConfiguration me = (ProducerConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.data);
	}

}

