package test.compiled.example6_stack;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class ClientPopBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Consume = 1;

	public static final short Pwait = 2;

	private final BroadcastChannel pop;

	private final BroadcastChannel ack;

	public ClientPopBehavior(SymbolsTable symbols, BroadcastChannel pop, BroadcastChannel ack) {
		super("clientPop", idSeed++, symbols);
		this.pop = pop;
		this.ack = ack;
	}

	private final String[] stateNames = new String[] {"consume", "pwait"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return pop.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ClientPopConfiguration me = (ClientPopConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Consume) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ClientPopConfiguration me = (ClientPopConfiguration) configuration.behaviorConfigurations[id];
			pop.output(id, actions);
			me.state = Pwait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return ack.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ClientPopConfiguration me = (ClientPopConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Pwait) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ClientPopConfiguration me = (ClientPopConfiguration) configuration.behaviorConfigurations[id];
			ack.input(id, actions);
			me.data = ack.value(int.class,0);
			if ( ack.clockId != -1 ) {
				DBM.set(configuration.dbm, ack.clockId, 0);
			}
			me.state = Consume;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ClientPopConfiguration me = new ClientPopConfiguration();
		//Initial action code.
		me.state = Consume;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ClientPopConfiguration me = (ClientPopConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ClientPopBehavior.Consume: 
			text.append("@consume,");
			break;
		case ClientPopBehavior.Pwait: 
			text.append("@pwait,");
			break;
		}
		text.append("\n\t- data=");
		text.append(Integer.toString(me.data));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ClientPopConfiguration me = (ClientPopConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ClientPopBehavior.Consume: 
			text.append("@consume,");
			break;
		case ClientPopBehavior.Pwait: 
			text.append("@pwait,");
			break;
		}
		text.append(" data=");
		text.append(Integer.toString(me.data));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ClientPopConfiguration me = new ClientPopConfiguration();
		me.state = boost.readShort();
		me.data = boost.readInt();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ClientPopConfiguration me = (ClientPopConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.data);
	}

}

