package test.compiled.example6_stack;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class ClientPushBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Start = 1;

	public static final short Produce = 2;

	public static final short Pwait = 3;

	private final BroadcastChannel stack_start;

	private final BroadcastChannel push;

	private final BroadcastChannel ack;

	public ClientPushBehavior(SymbolsTable symbols, BroadcastChannel stack_start, BroadcastChannel push, BroadcastChannel ack) {
		super("clientPush", idSeed++, symbols);
		this.stack_start = stack_start;
		this.push = push;
		this.ack = ack;
	}

	private final String[] stateNames = new String[] {"start", "produce", "pwait"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return stack_start.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ClientPushConfiguration me = (ClientPushConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ClientPushConfiguration me = (ClientPushConfiguration) configuration.behaviorConfigurations[id];
			stack_start.output(id, actions);
			me.state = Produce;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return push.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ClientPushConfiguration me = (ClientPushConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Produce) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ClientPushConfiguration me = (ClientPushConfiguration) configuration.behaviorConfigurations[id];
			push.output(id, actions, me.data);
			me.state = Pwait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return ack.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ClientPushConfiguration me = (ClientPushConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Pwait) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ClientPushConfiguration me = (ClientPushConfiguration) configuration.behaviorConfigurations[id];
			ack.input(id, actions);
			me.ackv = ack.value(boolean.class,0);
			if ( ack.clockId != -1 ) {
				DBM.set(configuration.dbm, ack.clockId, 0);
			}
			if (me.ackv == Example6_stackRoot.ACK) {
				me.data = ((me.data) + (1)) % (Example6_stackRoot.N);
				if (me.data == 0) {
					me.data = 1;
				}
			}
			me.state = Produce;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ClientPushConfiguration me = new ClientPushConfiguration();
		//Initial action code.
		me.data = 1;
		me.state = Start;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ClientPushConfiguration me = (ClientPushConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ClientPushBehavior.Start: 
			text.append("@start,");
			break;
		case ClientPushBehavior.Produce: 
			text.append("@produce,");
			break;
		case ClientPushBehavior.Pwait: 
			text.append("@pwait,");
			break;
		}
		text.append("\n\t- data=");
		text.append(Integer.toString(me.data));
		text.append("\n\t- ackv=");
		text.append(Boolean.toString(me.ackv));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ClientPushConfiguration me = (ClientPushConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ClientPushBehavior.Start: 
			text.append("@start,");
			break;
		case ClientPushBehavior.Produce: 
			text.append("@produce,");
			break;
		case ClientPushBehavior.Pwait: 
			text.append("@pwait,");
			break;
		}
		text.append(" data=");
		text.append(Integer.toString(me.data));
		text.append(" ackv=");
		text.append(Boolean.toString(me.ackv));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ClientPushConfiguration me = new ClientPushConfiguration();
		me.state = boost.readShort();
		me.data = boost.readInt();
		me.ackv = boost.readBoolean();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ClientPushConfiguration me = (ClientPushConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.data);
		boost.writeBoolean(me.ackv);
	}

}

