package test.compiled.example6_stack;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;

@SuppressWarnings("all")
public class StackBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Start = 1;

	public static final short Swait = 2;

	public static final short Wait_push = 3;

	public static final short Wait_pop = 4;

	public static final short Send_push_ack = 5;

	public static final short Send_pop_ack = 6;

	private final BroadcastChannel stack_start;

	private final BroadcastChannel pop;

	private final BroadcastChannel push;

	private final BroadcastChannel push_ack;

	private final BroadcastChannel pop_ack;

	public StackBehavior(SymbolsTable symbols, BroadcastChannel stack_start, BroadcastChannel pop, BroadcastChannel push, BroadcastChannel push_ack, BroadcastChannel pop_ack) {
		super("stack", idSeed++, symbols);
		this.stack_start = stack_start;
		this.pop = pop;
		this.push = push;
		this.push_ack = push_ack;
		this.pop_ack = pop_ack;
	}

	private final String[] stateNames = new String[] {"start", "swait", "wait_push", "wait_pop", "send_push_ack", "send_pop_ack"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return stack_start.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			stack_start.input(id, actions);
			if ( stack_start.clockId != -1 ) {
				DBM.set(configuration.dbm, stack_start.clockId, 0);
			}
			me.state = Swait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return stack_start.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Swait) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			stack_start.input(id, actions);
			if ( stack_start.clockId != -1 ) {
				DBM.set(configuration.dbm, stack_start.clockId, 0);
			}
			me.state = Swait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return push.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Swait) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			push.input(id, actions);
			me.data = push.value(int.class,0);
			if ( push.clockId != -1 ) {
				DBM.set(configuration.dbm, push.clockId, 0);
			}
			me.state = Wait_push;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition3 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return pop.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Swait) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			pop.input(id, actions);
			if ( pop.clockId != -1 ) {
				DBM.set(configuration.dbm, pop.clockId, 0);
			}
			me.state = Wait_pop;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition4 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Wait_push) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			if (me.top_index == (Example6_stackRoot.N) - (1)) {
				me.ack = Example6_stackRoot.NACK;
			} else {
				me.ack = Example6_stackRoot.ACK;
				me.top_index = (me.top_index) + (1);
				//Start copy variable 'me.stack'.
				{
					int[] __tmp;
					__tmp = new int[Example6_stackRoot.N];
					for (int k=0; k<me.stack.length; k++) {
						__tmp[k] = me.stack[k];
					}
					me.stack = __tmp;
				}
				//End copy variable 'me.stack'.
				me.stack[me.top_index] = me.data;
			}
			me.state = Send_push_ack;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition5 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Wait_pop) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			if (me.top_index ==  - (1)) {
				me.data =  - (1);
			} else {
				me.data = me.stack[me.top_index];
				me.top_index = (me.top_index) - (1);
			}
			me.state = Send_pop_ack;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition6 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return push_ack.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Send_push_ack) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			push_ack.output(id, actions, me.ack);
			me.state = Swait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition7 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return pop_ack.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Send_pop_ack) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final StackConfiguration me = (StackConfiguration) configuration.behaviorConfigurations[id];
			pop_ack.output(id, actions, me.data);
			me.state = Swait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
		transition3,
		transition4,
		transition5,
		transition6,
		transition7,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		StackConfiguration me = new StackConfiguration();
		//Initial action code.
		me.top_index =  - (1);
		me.state = Start;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		StackConfiguration me = (StackConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case StackBehavior.Start: 
			text.append("@start,");
			break;
		case StackBehavior.Swait: 
			text.append("@swait,");
			break;
		case StackBehavior.Wait_push: 
			text.append("@wait_push,");
			break;
		case StackBehavior.Wait_pop: 
			text.append("@wait_pop,");
			break;
		case StackBehavior.Send_push_ack: 
			text.append("@send_push_ack,");
			break;
		case StackBehavior.Send_pop_ack: 
			text.append("@send_pop_ack,");
			break;
		}
		text.append("\n\t- stack=");
		text.append(Arrays.toString(me.stack));
		text.append("\n\t- top_index=");
		text.append(Integer.toString(me.top_index));
		text.append("\n\t- ack=");
		text.append(Boolean.toString(me.ack));
		text.append("\n\t- data=");
		text.append(Integer.toString(me.data));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		StackConfiguration me = (StackConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case StackBehavior.Start: 
			text.append("@start,");
			break;
		case StackBehavior.Swait: 
			text.append("@swait,");
			break;
		case StackBehavior.Wait_push: 
			text.append("@wait_push,");
			break;
		case StackBehavior.Wait_pop: 
			text.append("@wait_pop,");
			break;
		case StackBehavior.Send_push_ack: 
			text.append("@send_push_ack,");
			break;
		case StackBehavior.Send_pop_ack: 
			text.append("@send_pop_ack,");
			break;
		}
		text.append(" stack=");
		text.append(Arrays.toString(me.stack));
		text.append(" top_index=");
		text.append(Integer.toString(me.top_index));
		text.append(" ack=");
		text.append(Boolean.toString(me.ack));
		text.append(" data=");
		text.append(Integer.toString(me.data));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		StackConfiguration me = new StackConfiguration();
		me.state = boost.readShort();
		me.stack = new int[Example6_stackRoot.N];
		for (int i=0; i<me.stack.length; i++) {
			me.stack[i] = boost.readInt();
		}
		me.top_index = boost.readInt();
		me.ack = boost.readBoolean();
		me.data = boost.readInt();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		StackConfiguration me = (StackConfiguration) processConfiguration;
		boost.writeShort(me.state);
		for (int i=0; i<me.stack.length; i++) {
			boost.writeInt(me.stack[i]);
		}
		boost.writeInt(me.top_index);
		boost.writeBoolean(me.ack);
		boost.writeInt(me.data);
	}

}

