package test.compiled.example7_mutex;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ReferencedArgumentInt;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class ProcWBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Start = 1;

	public static final short In_CS = 2;

	public static final short Out_CS = 3;

	private final BroadcastChannel mlock1;

	private final BroadcastChannel munlock1;

	private final ReferencedArgumentInt dataRef;

	public ProcWBehavior(SymbolsTable symbols, BroadcastChannel mlock1, BroadcastChannel munlock1, ReferencedArgumentInt data) {
		super("ProcW", idSeed++, symbols);
		this.mlock1 = mlock1;
		this.munlock1 = munlock1;
		this.dataRef = data;
	}

	private final String[] stateNames = new String[] {"Start", "In_CS", "Out_CS"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return mlock1.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcWConfiguration me = (ProcWConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcWConfiguration me = (ProcWConfiguration) configuration.behaviorConfigurations[id];
			mlock1.output(id, actions);
			me.state = In_CS;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProcWConfiguration me = (ProcWConfiguration) configuration.behaviorConfigurations[id];
			int data = dataRef.get(configuration);
			modifiedBehaviors[dataRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcWConfiguration me = (ProcWConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != In_CS) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcWConfiguration me = (ProcWConfiguration) configuration.behaviorConfigurations[id];
			int data = dataRef.get(configuration);
			try {
				data = me.data;
				me.data = ((me.data) + (1)) % (Example7_mutexRoot.N);
				me.state = Out_CS;
				if (true) return true;
			} finally {
				dataRef.set(configuration, data);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return munlock1.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcWConfiguration me = (ProcWConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Out_CS) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcWConfiguration me = (ProcWConfiguration) configuration.behaviorConfigurations[id];
			munlock1.output(id, actions);
			me.state = Start;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ProcWConfiguration me = new ProcWConfiguration();
		//Initial action code.
		me.data = 1;
		me.state = Start;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProcWConfiguration me = (ProcWConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProcWBehavior.Start: 
			text.append("@Start,");
			break;
		case ProcWBehavior.In_CS: 
			text.append("@In_CS,");
			break;
		case ProcWBehavior.Out_CS: 
			text.append("@Out_CS,");
			break;
		}
		text.append("\n\t- data=");
		text.append(Integer.toString(me.data));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProcWConfiguration me = (ProcWConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProcWBehavior.Start: 
			text.append("@Start,");
			break;
		case ProcWBehavior.In_CS: 
			text.append("@In_CS,");
			break;
		case ProcWBehavior.Out_CS: 
			text.append("@Out_CS,");
			break;
		}
		text.append(" data=");
		text.append(Integer.toString(me.data));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ProcWConfiguration me = new ProcWConfiguration();
		me.state = boost.readShort();
		me.data = boost.readInt();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ProcWConfiguration me = (ProcWConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.data);
	}

}

