package test.compiled.example7_semaphore;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class WBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Put3S = 1;

	public static final short Put2S = 2;

	public static final short EndS = 3;

	private final BroadcastChannel sv1;

	public WBehavior(SymbolsTable symbols, BroadcastChannel sv1) {
		super("w", idSeed++, symbols);
		this.sv1 = sv1;
	}

	private final String[] stateNames = new String[] {"put3S", "put2S", "endS"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return sv1.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			WConfiguration me = (WConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Put2S) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final WConfiguration me = (WConfiguration) configuration.behaviorConfigurations[id];
			sv1.output(id, actions, 2);
			me.state = Put3S;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return sv1.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			WConfiguration me = (WConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Put3S) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final WConfiguration me = (WConfiguration) configuration.behaviorConfigurations[id];
			sv1.output(id, actions, 3);
			me.state = EndS;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			WConfiguration me = (WConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != EndS) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final WConfiguration me = (WConfiguration) configuration.behaviorConfigurations[id];
			me.state = EndS;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		WConfiguration me = new WConfiguration();
		//Initial action code.
		me.state = Put2S;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		WConfiguration me = (WConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case WBehavior.Put3S: 
			text.append("@put3S,");
			break;
		case WBehavior.Put2S: 
			text.append("@put2S,");
			break;
		case WBehavior.EndS: 
			text.append("@endS,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		WConfiguration me = (WConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case WBehavior.Put3S: 
			text.append("@put3S,");
			break;
		case WBehavior.Put2S: 
			text.append("@put2S,");
			break;
		case WBehavior.EndS: 
			text.append("@endS,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		WConfiguration me = new WConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		WConfiguration me = (WConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

