package test.compiled.example_defer;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.FiacreQueueUtil;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ReferencedArgument;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class ProducerBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short S0 = 1;

	public static final short S1 = 2;

	private final ReferencedArgument<T_message[]> itsClientRef;

	public ProducerBehavior(SymbolsTable symbols, ReferencedArgument<T_message[]> itsClient) {
		super("producer", idSeed++, symbols);
		this.itsClientRef = itsClient;
	}

	private final String[] stateNames = new String[] {"s0", "s1"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != S0) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			me.state = S1;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			T_message[] itsClient = itsClientRef.get(configuration);
			modifiedBehaviors[itsClientRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != S1) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			T_message[] itsClient = itsClientRef.get(configuration);
			try {
				if ( !( (itsClient.length >= 2 )) == true) {
					itsClient = FiacreQueueUtil.enqueue(id, itsClient,new T_message.T_message_evtA(), actions);
				} else {
					if (true) { return false; }
				}
				me.state = S1;
				if (true) return true;
			} finally {
				itsClientRef.set(configuration, itsClient);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			T_message[] itsClient = itsClientRef.get(configuration);
			modifiedBehaviors[itsClientRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != S1) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProducerConfiguration me = (ProducerConfiguration) configuration.behaviorConfigurations[id];
			T_message[] itsClient = itsClientRef.get(configuration);
			try {
				if ( !( (itsClient.length >= 2 )) == true) {
					itsClient = FiacreQueueUtil.enqueue(id, itsClient,new T_message.T_message_evtB(), actions);
				} else {
					if (true) { return false; }
				}
				me.state = S1;
				if (true) return true;
			} finally {
				itsClientRef.set(configuration, itsClient);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ProducerConfiguration me = new ProducerConfiguration();
		//Initial action code.
		me.state = S0;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProducerConfiguration me = (ProducerConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProducerBehavior.S0: 
			text.append("@s0,");
			break;
		case ProducerBehavior.S1: 
			text.append("@s1,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProducerConfiguration me = (ProducerConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProducerBehavior.S0: 
			text.append("@s0,");
			break;
		case ProducerBehavior.S1: 
			text.append("@s1,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ProducerConfiguration me = new ProducerConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ProducerConfiguration me = (ProducerConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

