package test.compiled.example_fifo_ops;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.FiacreQueueUtil;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;

@SuppressWarnings("all")
public class PBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short S1 = 1;

	public static final short S0 = 2;

	public PBehavior(SymbolsTable symbols) {
		super("p", idSeed++, symbols);
	}

	private final String[] stateNames = new String[] {"s1", "s0"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != S0) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if ( (me.fifo.length == 0)) {
				me.fifo = FiacreQueueUtil.enqueue(me.fifo,2);
				me.state = S1;
				if (true) return true;
			} else {
				me.e =  FiacreQueueUtil.first(me.fifo);
				me.fifo =  FiacreQueueUtil.dequeue(me.fifo);
				me.state = S1;
				if (true) return true;
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != S1) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			me.fifo = FiacreQueueUtil.enqueue(me.fifo,3);
			me.state = S0;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		PConfiguration me = new PConfiguration();
		//Initial action code.
		me.state = S0;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		PConfiguration me = (PConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case PBehavior.S1: 
			text.append("@s1,");
			break;
		case PBehavior.S0: 
			text.append("@s0,");
			break;
		}
		text.append("\n\t- fifo=");
		text.append(Arrays.toString(me.fifo));
		text.append("\n\t- e=");
		text.append(Integer.toString(me.e));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		PConfiguration me = (PConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case PBehavior.S1: 
			text.append("@s1,");
			break;
		case PBehavior.S0: 
			text.append("@s0,");
			break;
		}
		text.append(" fifo=");
		text.append(Arrays.toString(me.fifo));
		text.append(" e=");
		text.append(Integer.toString(me.e));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		PConfiguration me = new PConfiguration();
		me.state = boost.readShort();
		me.fifo = new int[3];
		me.fifo = new int[boost.readInt()];
		for (int i=0; i<me.fifo.length; i++) {
			me.fifo[i] = boost.readInt();
		}
		me.e = boost.readInt();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		PConfiguration me = (PConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.fifo.length);
		for (int i=0; i<me.fifo.length; i++) {
			boost.writeInt(me.fifo[i]);
		}
		boost.writeInt(me.e);
	}

}

