package test.compiled.timer_flat;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class PBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Start = 1;

	public static final short WaitTA = 2;

	public static final short SB = 3;

	public static final short StopT = 4;

	private final BroadcastChannel t0_start;

	private final BroadcastChannel t0_stop;

	private final BroadcastChannel t0_timeout;

	public PBehavior(SymbolsTable symbols, BroadcastChannel t0_start, BroadcastChannel t0_stop, BroadcastChannel t0_timeout) {
		super("p", idSeed++, symbols);
		this.t0_start = t0_start;
		this.t0_stop = t0_stop;
		this.t0_timeout = t0_timeout;
	}

	private final String[] stateNames = new String[] {"start", "waitTA", "sB", "stopT"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return t0_start.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			t0_start.output(id, actions, 5);
			me.state = WaitTA;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return t0_timeout.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != WaitTA) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			t0_timeout.input(id, actions);
			if ( t0_timeout.clockId != -1 ) {
				DBM.set(configuration.dbm, t0_timeout.clockId, 0);
			}
			me.i = 2;
			me.state = SB;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return t0_start.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != SB) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			t0_start.output(id, actions, 2);
			me.state = StopT;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition3 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return t0_stop.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != StopT) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			t0_stop.output(id, actions);
			me.state = Start;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
		transition3,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		PConfiguration me = new PConfiguration();
		//Initial action code.
		me.i = 5;
		me.state = Start;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		PConfiguration me = (PConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case PBehavior.Start: 
			text.append("@start,");
			break;
		case PBehavior.WaitTA: 
			text.append("@waitTA,");
			break;
		case PBehavior.SB: 
			text.append("@sB,");
			break;
		case PBehavior.StopT: 
			text.append("@stopT,");
			break;
		}
		text.append("\n\t- i=");
		text.append(Integer.toString(me.i));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		PConfiguration me = (PConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case PBehavior.Start: 
			text.append("@start,");
			break;
		case PBehavior.WaitTA: 
			text.append("@waitTA,");
			break;
		case PBehavior.SB: 
			text.append("@sB,");
			break;
		case PBehavior.StopT: 
			text.append("@stopT,");
			break;
		}
		text.append(" i=");
		text.append(Integer.toString(me.i));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		PConfiguration me = new PConfiguration();
		me.state = boost.readShort();
		me.i = boost.readInt();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		PConfiguration me = (PConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.i);
	}

}

