package test.compiled.timer_flat;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.DelayAction;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.WaitChannel;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;

@SuppressWarnings("all")
public class TimerBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short SStart = 1;

	public static final short SWait = 2;

	public static final short STimeout = 3;

	private final BroadcastChannel start;

	private final BroadcastChannel stop;

	private final BroadcastChannel timeout;

	private final WaitChannel wait0;

	private final Channel[] channels;

	public void collectChannels(List<Channel> result) {
		result.addAll(Arrays.asList(channels));
	}

	public TimerBehavior(SymbolsTable symbols, BroadcastChannel start, BroadcastChannel stop, BroadcastChannel timeout) {
		super("Timer", idSeed++, symbols);
		this.start = start;
		this.stop = stop;
		this.timeout = timeout;
		symbols.registerClockIndexes(this.getId(), (short) 1);
		
		wait0 = new WaitChannel(null, symbols, this.getId(), (short) 0, 2, false, 2, false);
		channels = new Channel[] { wait0,  };
	}

	private final String[] stateNames = new String[] {"sStart", "sWait", "sTimeout"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return start.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			TimerConfiguration me = (TimerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != SStart) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final TimerConfiguration me = (TimerConfiguration) configuration.behaviorConfigurations[id];
			start.input(id, actions);
			me.tdElay = start.value(int.class,0);
			if ( start.clockId != -1 ) {
				DBM.set(configuration.dbm, start.clockId, 0);
			}
			me.state = SWait;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return wait0.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			TimerConfiguration me = (TimerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != SWait) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final TimerConfiguration me = (TimerConfiguration) configuration.behaviorConfigurations[id];
			DBM.set(configuration.dbm, wait0.clockId, 0);
			actions.add(new DelayAction(id));
			me.state = STimeout;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return stop.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			TimerConfiguration me = (TimerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != SWait) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final TimerConfiguration me = (TimerConfiguration) configuration.behaviorConfigurations[id];
			stop.input(id, actions);
			if ( stop.clockId != -1 ) {
				DBM.set(configuration.dbm, stop.clockId, 0);
			}
			me.state = SStart;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition3 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return timeout.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			TimerConfiguration me = (TimerConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != STimeout) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final TimerConfiguration me = (TimerConfiguration) configuration.behaviorConfigurations[id];
			timeout.output(id, actions);
			me.state = SStart;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
		transition3,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		TimerConfiguration me = new TimerConfiguration();
		//Initial action code.
		me.state = SStart;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		TimerConfiguration me = (TimerConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case TimerBehavior.SStart: 
			text.append("@sStart,");
			break;
		case TimerBehavior.SWait: 
			text.append("@sWait,");
			break;
		case TimerBehavior.STimeout: 
			text.append("@sTimeout,");
			break;
		}
		text.append("\n\t- tdElay=");
		text.append(Integer.toString(me.tdElay));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		TimerConfiguration me = (TimerConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case TimerBehavior.SStart: 
			text.append("@sStart,");
			break;
		case TimerBehavior.SWait: 
			text.append("@sWait,");
			break;
		case TimerBehavior.STimeout: 
			text.append("@sTimeout,");
			break;
		}
		text.append(" tdElay=");
		text.append(Integer.toString(me.tdElay));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		TimerConfiguration me = new TimerConfiguration();
		me.state = boost.readShort();
		me.tdElay = boost.readInt();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		TimerConfiguration me = (TimerConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.tdElay);
	}

}

