package test.compiled.timer_shared;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class P1Behavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Ende = 1;

	public static final short Starte = 2;

	private final BroadcastChannel stop1;

	private final BroadcastChannel timeout1;

	public P1Behavior(SymbolsTable symbols, BroadcastChannel stop1, BroadcastChannel timeout1) {
		super("p1", idSeed++, symbols);
		this.stop1 = stop1;
		this.timeout1 = timeout1;
	}

	private final String[] stateNames = new String[] {"ende", "starte"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return timeout1.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			P1Configuration me = (P1Configuration) configuration.behaviorConfigurations[id];
			if (me.state != Starte) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final P1Configuration me = (P1Configuration) configuration.behaviorConfigurations[id];
			timeout1.input(id, actions);
			if ( timeout1.clockId != -1 ) {
				DBM.set(configuration.dbm, timeout1.clockId, 0);
			}
			me.state = Ende;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return stop1.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			P1Configuration me = (P1Configuration) configuration.behaviorConfigurations[id];
			if (me.state != Starte) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final P1Configuration me = (P1Configuration) configuration.behaviorConfigurations[id];
			me.i = ((me.i) + (1)) % (2);
			stop1.output(id, actions);
			me.state = Starte;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		P1Configuration me = new P1Configuration();
		//Initial action code.
		me.state = Starte;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		P1Configuration me = (P1Configuration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case P1Behavior.Ende: 
			text.append("@ende,");
			break;
		case P1Behavior.Starte: 
			text.append("@starte,");
			break;
		}
		text.append("\n\t- i=");
		text.append(Integer.toString(me.i));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		P1Configuration me = (P1Configuration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case P1Behavior.Ende: 
			text.append("@ende,");
			break;
		case P1Behavior.Starte: 
			text.append("@starte,");
			break;
		}
		text.append(" i=");
		text.append(Integer.toString(me.i));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		P1Configuration me = new P1Configuration();
		me.state = boost.readShort();
		me.i = boost.readInt();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		P1Configuration me = (P1Configuration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.i);
	}

}

