package test.compiled.timer_shared;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;

@SuppressWarnings("all")
public class PBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short WaitTA = 1;

	public static final short Start = 2;

	private final BroadcastChannel start1;

	private final BroadcastChannel timeout1;

	public PBehavior(SymbolsTable symbols, BroadcastChannel start1, BroadcastChannel timeout1) {
		super("p", idSeed++, symbols);
		this.start1 = start1;
		this.timeout1 = timeout1;
	}

	private final String[] stateNames = new String[] {"waitTA", "start"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return start1.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Start) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			start1.output(id, actions, 5);
			me.state = WaitTA;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return timeout1.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != WaitTA) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PConfiguration me = (PConfiguration) configuration.behaviorConfigurations[id];
			timeout1.input(id, actions);
			if ( timeout1.clockId != -1 ) {
				DBM.set(configuration.dbm, timeout1.clockId, 0);
			}
			me.state = Start;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		PConfiguration me = new PConfiguration();
		//Initial action code.
		me.state = Start;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		PConfiguration me = (PConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case PBehavior.WaitTA: 
			text.append("@waitTA,");
			break;
		case PBehavior.Start: 
			text.append("@start,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		PConfiguration me = (PConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case PBehavior.WaitTA: 
			text.append("@waitTA,");
			break;
		case PBehavior.Start: 
			text.append("@start,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		PConfiguration me = new PConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		PConfiguration me = (PConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

